@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## Cookies API for Server & Client - ~/lib/Cookie.pm
-## Version v0.1.2
+## Version v0.1.4
 ## Copyright(c) 2021 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2019/10/08
-## Modified 2021/11/18
+## Modified 2021/11/22
 ## You can use, copy, modify and  redistribute  this  package  and  associated
 ## files under the same terms as Perl itself.
 ##----------------------------------------------------------------------------
@@ -27,7 +27,7 @@ BEGIN
         '=='     => \&same_as,
         fallback => 1,
     );
-    our $VERSION = 'v0.1.2';
+    our $VERSION = 'v0.1.4';
     our $SUBS;
     our $COOKIE_DEBUG = 0;
 };
@@ -331,9 +331,16 @@ sub expires
         }
         elsif( $exp =~ /^\d{1,10}$/ )
         {
-            $self->message( 3, "Value is actually a unix timestamp." ) if( $COOKIE_DEBUG );
+            $self->message( 3, "Value is actually a unix timestamp '$exp' -> ", scalar( localtime( $exp ) ), "." ) if( $COOKIE_DEBUG );
             try
             {
+                # Unexpectedly, DateTime sets the time zone ONLY after having instantiated the
+                # object and set its time zone to UTC.
+                # Thus, here setting to 'local' (e.g. corresponding to Asia/Tokyo) would
+                # actually set the epoch to GMT+9 instead of treating the epoch time provided
+                # to being in Asia/Tokyo time zone!
+                # Issue #126
+                # <https://github.com/houseabsolute/DateTime.pm/issues/126>
                 $dt = DateTime->from_epoch( epoch => $exp, time_zone => 'local' );
             }
             catch( $e )
@@ -368,7 +375,7 @@ sub expires
         elsif( lc( $exp ) eq 'now' )
         {
             # $self->message( 3, "Value is actually a special keyword." );
-            $dt = DateTime->now;
+            $dt = DateTime->now( time_zone => 'local' );
         }
         elsif( defined( $exp ) && CORE::length( $exp ) )
         {
@@ -410,12 +417,25 @@ sub is_expired
 {
     my $self = shift( @_ );
     my $exp = $self->expires;
+    my $max_age = $self->max_age;
     # $self->message( 4, "Expiration value is '", ( $exp // '' ), "' (", overload::StrVal( $exp ), ")." );
-    return( $self->false ) if( !defined( $exp ) );
-    return( $self->false ) if( !$self->_is_a( $exp, 'Module::Generic::DateTime' ) && !$self->_is_a( $exp, 'DateTime' ) );
+    return( $self->false ) if( !defined( $exp ) && !defined( $max_age ) );
+    if( ( defined( $exp ) && !$self->_is_a( $exp, 'Module::Generic::DateTime' ) && !$self->_is_a( $exp, 'DateTime' ) ) ||
+        ( defined( $max_age ) && $max_age !~ /\-?\d+$/ ) )
+    {
+        return( $self->false );
+    }
     my $now = DateTime->now;
     # $self->message( 4, "Is $exp lower than $now ? ", ( $exp < $now ? 'yes' : 'no' ) );
-    return( $exp < $now ? $self->true : $self->false );
+    if( ( defined( $max_age ) && $max_age <= 0 ) || 
+        ( defined( $exp ) && $exp < $now ) )
+    {
+        return( $self->true );
+    }
+    else
+    {
+        return( $self->false );
+    }
 }
 
 sub is_persistent { return( !shift->is_session ); }
@@ -423,7 +443,7 @@ sub is_persistent { return( !shift->is_session ); }
 sub is_session
 {
     my $self = shift( @_ );
-    return( defined( $self->expires ) ? $self->false : $self->true );
+    return( defined( $self->expires ) || defined( $self->max_age ) ? $self->false : $self->true );
 }
 
 sub is_tainted { return( shift->_set_get_boolean( 'is_tainted', @_ ) ); }
@@ -690,11 +710,12 @@ sub _header_datetime
         return( $self->error( "Object provided (", ref( $_[0] ), ") is not a DateTime object." ) ) if( !$_[0]->isa( 'DateTime' ) );
         $dt = shift( @_ );
     }
-    $dt = DateTime->now if( !defined( $dt ) );
+    $dt = DateTime->now( time_zone => 'local' ) if( !defined( $dt ) );
+    $dt->set_time_zone( 'GMT' );
     my $fmt = DateTime::Format::Strptime->new(
         pattern => '%a, %d %b %Y %H:%M:%S GMT',
         locale  => 'en_GB',
-        time_zone => 'local',
+        time_zone => 'GMT',
     );
     $dt->set_formatter( $fmt );
     return( $dt );
@@ -721,7 +742,7 @@ __END__
 
 =head1 NAME
 
-Cookie - Cookie Object
+Cookie - Cookie Object with Encryption or Signature
 
 =head1 SYNOPSIS
 
@@ -734,7 +755,7 @@ Cookie - Cookie Object
         expires => '+10D',
         # or alternatively
         maxage => 864000
-        # to make it exclusively accessible by regular http request and not ajax
+        # to make it exclusively accessible by regular http request and not javascript
         http_only => 1,
         same_site => 'Lax',
         # should it be used under ssl only?
@@ -814,7 +835,7 @@ Cookie - Cookie Object
 
 =head1 VERSION
 
-    v0.1.2
+    v0.1.4
 
 =head1 DESCRIPTION
 
@@ -827,7 +848,7 @@ The object is overloaded and will call L</as_string> upon stringification and ca
         # do something
     }
 
-This module does not die upon error, but instead returns C<undef> and sets an L<eror|Module::Generic/error>, so you should always check the return value of a method.
+This module does not die upon error, but instead returns C<undef> and sets an L<error|Module::Generic/error>, so you should always check the return value of a method.
 
 See also the L<Cookie::Jar> package to manage server and client side handling of cookies:
 
@@ -921,7 +942,7 @@ This set or get the the algorithm used to encrypt the cookie value.
 
 It can be any of L<AES|Crypt::Cipher::AES>, L<Anubis|Crypt::Cipher::Anubis>, L<Blowfish|Crypt::Cipher::Blowfish>, L<CAST5|Crypt::Cipher::CAST5>, L<Camellia|Crypt::Cipher::Camellia>, L<DES|Crypt::Cipher::DES>, L<DES_EDE|Crypt::Cipher::DES_EDE>, L<KASUMI|Crypt::Cipher::KASUMI>, L<Khazad|Crypt::Cipher::Khazad>, L<MULTI2|Crypt::Cipher::MULTI2>, L<Noekeon|Crypt::Cipher::Noekeon>, L<RC2|Crypt::Cipher::RC2>, L<RC5|Crypt::Cipher::RC5>, L<RC6|Crypt::Cipher::RC6>, L<SAFERP|Crypt::Cipher::SAFERP>, L<SAFER_K128|Crypt::Cipher::SAFER_K128>, L<SAFER_K64|Crypt::Cipher::SAFER_K64>, L<SAFER_SK128|Crypt::Cipher::SAFER_SK128>, L<SAFER_SK64|Crypt::Cipher::SAFER_SK64>, L<SEED|Crypt::Cipher::SEED>, L<Skipjack|Crypt::Cipher::Skipjack>, L<Twofish|Crypt::Cipher::Twofish>, L<XTEA|Crypt::Cipher::XTEA>, L<IDEA|Crypt::Cipher::IDEA>, L<Serpent|Crypt::Cipher::Serpent> or simply any <NAME> for which there exists Crypt::Cipher::<NAME>
 
-See also L<Stackoverflow> on the choice of encryption algorithm|https://stackoverflow.com/questions/4147451/aes-vs-blowfish-for-file-encryption>
+See also L<Stackoverflow on the choice of encryption algorithm|https://stackoverflow.com/questions/4147451/aes-vs-blowfish-for-file-encryption>
 
 By default, the algorithm is set to C<AES>
 
@@ -1084,7 +1105,7 @@ However, this is not sprintf, so you cannot combine them, thus B<you cannot do t
 
 Special keyword
 
-=item In last resort, the value provided will be parsed using L<Module::Generic/_parse_datetime>. If parsing fails, it will return C<undef> and set an error.
+=item In last resort, the value provided will be parsed using L<Module::Generic/_parse_timestamp>. If parsing fails, it will return C<undef> and set an error.
 
 =back
 
@@ -1151,13 +1172,13 @@ Return value is in the form of a L<Module::Generic::Boolean> object that stringi
 
 =head2 is_persistent
 
-Boolean. This returns true if the cookie sent from the server is not a session cookie, i.e. it has an C</expires> value set.
+Boolean. This returns true if the cookie sent from the server is not a session cookie, i.e. it has an L</expires> value set.
 
 See L<rfc62655, section 5.3.3|https://datatracker.ietf.org/doc/html/rfc6265#section-5.3>
 
 =head2 is_session
 
-Returns true if this is a session cookie, i.e. it has no expiration datetime set, otherwise, it returns false.
+Returns true if this is a session cookie, i.e. it has no expiration datetime nor any L</max_age> set, otherwise, it returns false.
 
 Return value is in the form of a L<Module::Generic::Boolean> object that stringifies to 1 or 0;
 
@@ -1169,7 +1190,7 @@ Returns a L<Module::Generic::Boolean> object.
 
 =head2 is_valid
 
-This taks an optional hash or hash reference of parameters.
+This takes an optional hash or hash reference of parameters.
 
 It returns true if the cookie was signed and the signature is valid, or false otherwise.
 
@@ -1181,7 +1202,7 @@ If an error occurred, this method returns C<undef> and sets an L<error|Module::G
 
 Return value is in the form of a L<Module::Generic::Boolean> object that stringifies to 1 or 0;
 
-Possibly parameters are:
+Possible parameters are:
 
 =over 4
 
